@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2019 pyglet contributors
+# Copyright (c) 2008-2020 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,26 +33,19 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
 
-'''
-'''
-
-__docformat__ = 'restructuredtext'
-__version__ = '$Id$'
-
 from ctypes import *
 
 import pyglet.lib
 from pyglet.gl.lib import missing_function, decorate_function
 
-from pyglet.compat import asbytes
+from pyglet.util import asbytes
 
 __all__ = ['link_GL', 'link_GLU', 'link_GLX']
 
 gl_lib = pyglet.lib.load_library('GL')
 glu_lib = pyglet.lib.load_library('GLU')
 
-# Look for glXGetProcAddressARB extension, use it as fallback (for
-# ATI fglrx and DRI drivers).
+# Look for glXGetProcAddressARB extension, use it as fallback (for ATI fglrx and DRI drivers).
 try:
     glXGetProcAddressARB = getattr(gl_lib, 'glXGetProcAddressARB')
     glXGetProcAddressARB.restype = POINTER(CFUNCTYPE(None))
@@ -60,7 +53,8 @@ try:
     _have_getprocaddress = True
 except AttributeError:
     _have_getprocaddress = False
-    
+
+
 def link_GL(name, restype, argtypes, requires=None, suggestions=None):
     try:
         func = getattr(gl_lib, name)
@@ -81,8 +75,10 @@ def link_GL(name, restype, argtypes, requires=None, suggestions=None):
 
     return missing_function(name, requires, suggestions)
 
+
 link_GLX = link_GL
 
+
 def link_GLU(name, restype, argtypes, requires=None, suggestions=None):
     try:
         func = getattr(glu_lib, name)
@@ -92,4 +88,3 @@ def link_GLU(name, restype, argtypes, requires=None, suggestions=None):
         return func
     except AttributeError:
         return missing_function(name, requires, suggestions)
-
