@@ -1,6 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
+# Copyright (c) 2008-2020 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,35 +33,36 @@
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
 
-'''Load application resources from a known path.
+"""Load application resources from a known path.
 
 Loading resources by specifying relative paths to filenames is often
 problematic in Python, as the working directory is not necessarily the same
 directory as the application's script files.
 
 This module allows applications to specify a search path for resources.
-Relative paths are taken to be relative to the application's __main__ module.
-ZIP files can appear on the path; they will be searched inside.  The resource
-module also behaves as expected when applications are bundled using py2exe or
-py2app.
+Relative paths are taken to be relative to the application's ``__main__``
+module. ZIP files can appear on the path; they will be searched inside.  The
+resource module also behaves as expected when applications are bundled using
+Freezers such as PyInstaller, py2exe, py2app, etc..
 
-As well as providing file references (with the `file` function), the resource
-module also contains convenience functions for loading images, textures,
-fonts, media and documents.
+In addition to providing file references (with the :py:func:`file` function),
+the resource module also contains convenience functions for loading images,
+textures, fonts, media and documents.
 
 3rd party modules or packages not bound to a specific application should
-construct their own `Loader` instance and override the path to use the
+construct their own :py:class:`Loader` instance and override the path to use the
 resources in the module's directory.
 
 Path format
 ^^^^^^^^^^^
 
-The resource path `path` (see also `Loader.__init__` and `Loader.path`)
+The resource path :py:attr:`path` (see also :py:meth:`Loader.__init__` and
+:py:meth:`Loader.path`)
 is a list of locations to search for resources.  Locations are searched in the
 order given in the path.  If a location is not valid (for example, if the
 directory does not exist), it is skipped.
 
-Locations in the path beginning with an ampersand (''@'' symbol) specify
+Locations in the path beginning with an "at" symbol (''@'') specify
 Python packages.  Other locations specify a ZIP archive or directory on the
 filesystem.  Locations that are not absolute are assumed to be relative to the
 script home.  Some examples::
@@ -73,37 +75,36 @@ script home.  Some examples::
     # by the `res/images` directory.
     path = ['@levels.level1', 'res/images']
 
-Paths are always case-sensitive and forward slashes are always used as path
-separators, even in cases when the filesystem or platform does not do this.
-This avoids a common programmer error when porting applications between
-platforms.
+Paths are always **case-sensitive** and **forward slashes are always used**
+as path separators, even in cases when the filesystem or platform does not do this.
+This avoids a common programmer error when porting applications between platforms.
 
 The default path is ``['.']``.  If you modify the path, you must call
-`reindex`.
+:py:func:`reindex`.
 
-:since: pyglet 1.1
-'''
-
-__docformat__ = 'restructuredtext'
-__version__ = '$Id: $'
+.. versionadded:: 1.1
+"""
 
+import io
 import os
-import weakref
 import sys
 import zipfile
+import weakref
 
 import pyglet
-from pyglet.compat import BytesIO
+
 
 class ResourceNotFoundException(Exception):
-    '''The named resource was not found on the search path.'''
+    """The named resource was not found on the search path."""
+
     def __init__(self, name):
         message = ('Resource "%s" was not found on the path.  '
-            'Ensure that the filename has the correct captialisation.') % name
+                   'Ensure that the filename has the correct captialisation.') % name
         Exception.__init__(self, message)
 
+
 def get_script_home():
-    '''Get the directory containing the program entry module.
+    """Get the directory containing the program entry module.
 
     For ordinary Python scripts, this is the directory containing the
     ``__main__`` module.  For executables created with py2exe the result is
@@ -117,13 +118,16 @@ def get_script_home():
     When the script is being run by a Python profiler, this function
     may return the directory where the profiler is running instead of
     the directory of the real script. To workaround this behaviour the
-    full path to the real script can be specified in `pyglet.resource.path`.
+    full path to the real script can be specified in :py:attr:`pyglet.resource.path`.
 
     :rtype: str
-    '''
-
+    """
     frozen = getattr(sys, 'frozen', None)
-    if frozen in ('windows_exe', 'console_exe'):
+    meipass = getattr(sys, '_MEIPASS', None)
+    if meipass:
+        # PyInstaller
+        return meipass
+    elif frozen in ('windows_exe', 'console_exe'):
         return os.path.dirname(sys.executable)
     elif frozen == 'macosx_app':
         # py2app
@@ -133,14 +137,16 @@ def get_script_home():
         if hasattr(main, '__file__'):
             return os.path.dirname(os.path.abspath(main.__file__))
         else:
-            # cx_Freeze
-            return os.path.dirname(sys.executable)
+            if 'python' in os.path.basename(sys.executable):
+                # interactive
+                return os.getcwd()
+            else:
+                # cx_Freeze
+                return os.path.dirname(sys.executable)
 
-    # Probably interactive
-    return ''
 
 def get_settings_path(name):
-    '''Get a directory to save user preferences.
+    """Get a directory to save user preferences.
 
     Different platforms have different conventions for where to save user
     preferences, saved games, and settings.  This function implements those
@@ -161,7 +167,7 @@ def get_settings_path(name):
             The name of the application.
 
     :rtype: str
-    '''
+    """
 
     if pyglet.compat_platform in ('cygwin', 'win32'):
         if 'APPDATA' in os.environ:
@@ -178,15 +184,17 @@ def get_settings_path(name):
     else:
         return os.path.expanduser('~/.%s' % name)
 
-class Location(object):
-    '''Abstract resource location.
+
+class Location:
+    """Abstract resource location.
 
     Given a location, a file can be loaded from that location with the `open`
     method.  This provides a convenient way to specify a path to load files
     from, and not necessarily have that path reside on the filesystem.
-    '''
+    """
+
     def open(self, filename, mode='rb'):
-        '''Open a file at this location.
+        """Open a file at this location.
 
         :Parameters:
             `filename` : str
@@ -198,29 +206,33 @@ class Location(object):
                 filesystem make use of this parameter; others ignore it.
 
         :rtype: file object
-        '''
+        """
         raise NotImplementedError('abstract')
 
+
 class FileLocation(Location):
-    '''Location on the filesystem.
-    '''
+    """Location on the filesystem.
+    """
+
     def __init__(self, path):
-        '''Create a location given a relative or absolute path.
+        """Create a location given a relative or absolute path.
 
         :Parameters:
             `path` : str
                 Path on the filesystem.
-        '''
+        """
         self.path = path
 
     def open(self, filename, mode='rb'):
         return open(os.path.join(self.path, filename), mode)
 
+
 class ZIPLocation(Location):
-    '''Location within a ZIP file.
-    '''
+    """Location within a ZIP file.
+    """
+
     def __init__(self, zip, dir):
-        '''Create a location given an open ZIP file and a path within that
+        """Create a location given an open ZIP file and a path within that
         file.
 
         :Parameters:
@@ -230,7 +242,7 @@ class ZIPLocation(Location):
                 A path within that ZIP file.  Can be empty to specify files at
                 the top level of the ZIP file.
 
-        '''
+        """
         self.zip = zip
         self.dir = dir
 
@@ -239,33 +251,37 @@ class ZIPLocation(Location):
             path = self.dir + '/' + filename
         else:
             path = filename
-        text = self.zip.read(path)
-        return BytesIO(text)
+
+        forward_slash_path = path.replace(os.sep, '/')  # zip can only handle forward slashes
+        text = self.zip.read(forward_slash_path)
+        return io.BytesIO(text)
+
 
 class URLLocation(Location):
-    '''Location on the network.
+    """Location on the network.
 
     This class uses the ``urlparse`` and ``urllib2`` modules to open files on
     the network given a URL.
-    '''
+    """
+
     def __init__(self, base_url):
-        '''Create a location given a base URL.
+        """Create a location given a base URL.
 
         :Parameters:
             `base_url` : str
                 URL string to prepend to filenames.
 
-        '''
+        """
         self.base = base_url
 
     def open(self, filename, mode='rb'):
-        import urllib.parse
-        import urllib.request, urllib.error, urllib.parse
+        import urllib.parse, urllib.request
         url = urllib.parse.urljoin(self.base, filename)
         return urllib.request.urlopen(url)
 
-class Loader(object):
-    '''Load program resource files from disk.
+
+class Loader:
+    """Load program resource files from disk.
 
     The loader contains a search path which can include filesystem
     directories, ZIP archives and Python packages.
@@ -278,9 +294,9 @@ class Loader(object):
             Base resource location, defaulting to the location of the
             application script.
 
-    '''
+    """
     def __init__(self, path=None, script_home=None):
-        '''Create a loader for the given path.
+        """Create a loader for the given path.
 
         If no path is specified it defaults to ``['.']``; that is, just the
         program directory.
@@ -294,35 +310,33 @@ class Loader(object):
                 Base location of relative files.  Defaults to the result of
                 `get_script_home`.
 
-        '''
+        """
         if path is None:
             path = ['.']
-        if type(path) in (str, str):
+        if isinstance(path, str):
             path = [path]
         self.path = list(path)
-        if script_home is None:
-            script_home = get_script_home()
-        self._script_home = script_home
+        self._script_home = script_home or get_script_home()
         self._index = None
 
         # Map bin size to list of atlases
         self._texture_atlas_bins = {}
 
+        # map name to image etc.
+        self._cached_textures = weakref.WeakValueDictionary()
+        self._cached_images = weakref.WeakValueDictionary()
+        self._cached_animations = weakref.WeakValueDictionary()
+
     def _require_index(self):
         if self._index is None:
             self.reindex()
 
     def reindex(self):
-        '''Refresh the file index.
+        """Refresh the file index.
 
         You must call this method if `path` is changed or the filesystem
         layout changes.
-        '''
-        # map name to image etc.
-        self._cached_textures = weakref.WeakValueDictionary()
-        self._cached_images = weakref.WeakValueDictionary()
-        self._cached_animations = weakref.WeakValueDictionary()
-
+        """
         self._index = {}
         for path in self.path:
             if path.startswith('@'):
@@ -340,11 +354,10 @@ class Loader(object):
                 if hasattr(module, '__file__'):
                     path = os.path.dirname(module.__file__)
                 else:
-                    path = '' # interactive
+                    path = ''  # interactive
             elif not os.path.isabs(path):
                 # Add script base unless absolute
-                assert '\\' not in path, \
-                    'Backslashes not permitted in relative path'
+                assert r'\\' not in path, "Backslashes are not permitted in relative paths"
                 path = os.path.join(self._script_home, path)
 
             if os.path.isdir(path):
@@ -355,7 +368,10 @@ class Loader(object):
                     dirpath = dirpath[len(path) + 1:]
                     # Force forward slashes for index
                     if dirpath:
-                        parts = [_f for _f in dirpath.split(os.sep) if _f]
+                        parts = [part
+                                 for part
+                                 in dirpath.split(os.sep)
+                                 if part is not None]
                         dirpath = '/'.join(parts)
                     for filename in filenames:
                         if dirpath:
@@ -364,10 +380,10 @@ class Loader(object):
                             index_name = filename
                         self._index_file(index_name, location)
             else:
-                # Find path component that is the ZIP file.
+                # Find path component that looks like the ZIP file.
                 dir = ''
                 old_path = None
-                while path and not os.path.isfile(path):
+                while path and not (os.path.isfile(path) or os.path.isfile(path + '.001')):
                     old_path = path
                     path, tail_dir = os.path.split(path)
                     if path == old_path:
@@ -377,25 +393,51 @@ class Loader(object):
                     continue
                 dir = dir.rstrip('/')
 
-                # path is a ZIP file, dir resides within ZIP
-                if path and zipfile.is_zipfile(path):
-                    zip = zipfile.ZipFile(path, 'r')
+                # path looks like a ZIP file, dir resides within ZIP
+                if not path:
+                    continue
+
+                zip_stream = self._get_stream(path)
+                if zip_stream:
+                    zip = zipfile.ZipFile(zip_stream, 'r')
                     location = ZIPLocation(zip, dir)
                     for zip_name in zip.namelist():
-                        #zip_name_dir, zip_name = os.path.split(zip_name)
-                        #assert '\\' not in name_dir
-                        #assert not name_dir.endswith('/')
+                        # zip_name_dir, zip_name = os.path.split(zip_name)
+                        # assert '\\' not in name_dir
+                        # assert not name_dir.endswith('/')
                         if zip_name.startswith(dir):
                             if dir:
-                                zip_name = zip_name[len(dir)+1:]
+                                zip_name = zip_name[len(dir) + 1:]
                             self._index_file(zip_name, location)
 
+    def _get_stream(self, path):
+        if zipfile.is_zipfile(path):
+            return path
+        elif not os.path.exists(path + '.001'):
+            return None
+        else:
+            with open(path + '.001', 'rb') as volume:
+                bytes_ = bytes(volume.read())
+
+            volume_index = 2
+            while os.path.exists(path + '.{0:0>3}'.format(volume_index)):
+                with open(path + '.{0:0>3}'.format(volume_index), 'rb') as volume:
+                    bytes_ += bytes(volume.read())
+
+                volume_index += 1
+
+            zip_stream = io.BytesIO(bytes_)
+            if zipfile.is_zipfile(zip_stream):
+                return zip_stream
+            else:
+                return None
+
     def _index_file(self, name, location):
         if name not in self._index:
             self._index[name] = location
 
     def file(self, name, mode='rb'):
-        '''Load a resource.
+        """Load a resource.
 
         :Parameters:
             `name` : str
@@ -405,7 +447,7 @@ class Loader(object):
                 with the meaning as for the builtin ``open`` function.
 
         :rtype: file object
-        '''
+        """
         self._require_index()
         try:
             location = self._index[name]
@@ -414,7 +456,7 @@ class Loader(object):
             raise ResourceNotFoundException(name)
 
     def location(self, name):
-        '''Get the location of a resource.
+        """Get the location of a resource.
 
         This method is useful for opening files referenced from a resource.
         For example, an HTML file loaded as a resource might reference some
@@ -426,7 +468,7 @@ class Loader(object):
                 Filename of the resource to locate.
 
         :rtype: `Location`
-        '''
+        """
         self._require_index()
         try:
             return self._index[name]
@@ -434,7 +476,7 @@ class Loader(object):
             raise ResourceNotFoundException(name)
 
     def add_font(self, name):
-        '''Add a font resource to the application.
+        """Add a font resource to the application.
 
         Fonts not installed on the system must be added to pyglet before they
         can be used with `font.load`.  Although the font is added with
@@ -448,7 +490,7 @@ class Loader(object):
             `name` : str
                 Filename of the font resource to add.
 
-        '''
+        """
         self._require_index()
         from pyglet import font
         file = self.file(name)
@@ -472,33 +514,35 @@ class Loader(object):
         return bin.add(img)
 
     def _get_texture_atlas_bin(self, width, height):
-        '''A heuristic for determining the atlas bin to use for a given image
+        """A heuristic for determining the atlas bin to use for a given image
         size.  Returns None if the image should not be placed in an atlas (too
         big), otherwise the bin (a list of TextureAtlas).
-        '''
+        """
         # Large images are not placed in an atlas
-        if width > 128 or height > 128:
+        max_texture_size = pyglet.image.get_max_texture_size()
+        max_size = min(2048, max_texture_size)
+        if width > max_size or height > max_size:
             return None
 
-        # Group images with small height separately to larger height (as the
-        # allocator can't stack within a single row).
+        # Group images with small height separately to larger height
+        # (as the allocator can't stack within a single row).
         bin_size = 1
-        if height > 32:
+        if height > max_size / 4:
             bin_size = 2
 
         try:
-            bin = self._texture_atlas_bins[bin_size]
+            texture_bin = self._texture_atlas_bins[bin_size]
         except KeyError:
-            bin = self._texture_atlas_bins[bin_size] = \
-                pyglet.image.atlas.TextureBin()
+            texture_bin = pyglet.image.atlas.TextureBin(border=True)
+            self._texture_atlas_bins[bin_size] = texture_bin
 
-        return bin
+        return texture_bin
 
     def image(self, name, flip_x=False, flip_y=False, rotate=0, atlas=True):
-        '''Load an image with optional transformation.
+        """Load an image with optional transformation.
 
         This is similar to `texture`, except the resulting image will be
-        packed into a `TextureBin` if it is an appropriate size for packing.
+        packed into a :py:class:`~pyglet.image.atlas.TextureBin` if it is an appropriate size for packing.
         This is more efficient than loading images into separate textures.
 
         :Parameters:
@@ -519,14 +563,13 @@ class Loader(object):
 
         :rtype: `Texture`
         :return: A complete texture if the image is large or not in an atlas,
-            otherwise a `TextureRegion` of a texture atlas.
-        '''
+            otherwise a :py:class:`~pyglet.image.TextureRegion` of a texture atlas.
+        """
         self._require_index()
         if name in self._cached_images:
             identity = self._cached_images[name]
         else:
-            identity = self._cached_images[name] = self._alloc_image(name,
-                atlas=atlas)
+            identity = self._cached_images[name] = self._alloc_image(name, atlas=atlas)
 
         if not rotate and not flip_x and not flip_y:
             return identity
@@ -534,7 +577,7 @@ class Loader(object):
         return identity.get_transform(flip_x, flip_y, rotate)
 
     def animation(self, name, flip_x=False, flip_y=False, rotate=0):
-        '''Load an animation with optional transformation.
+        """Load an animation with optional transformation.
 
         Animations loaded from the same source but with different
         transformations will use the same textures.
@@ -550,8 +593,8 @@ class Loader(object):
                 The returned image will be rotated clockwise by the given
                 number of degrees (a multiple of 90).
 
-        :rtype: `Animation`
-        '''
+        :rtype: :py:class:`~pyglet.image.Animation`
+        """
         self._require_index()
         try:
             identity = self._cached_animations[name]
@@ -570,40 +613,40 @@ class Loader(object):
         return identity.get_transform(flip_x, flip_y, rotate)
 
     def get_cached_image_names(self):
-        '''Get a list of image filenames that have been cached.
+        """Get a list of image filenames that have been cached.
 
         This is useful for debugging and profiling only.
 
         :rtype: list
         :return: List of str
-        '''
+        """
         self._require_index()
         return list(self._cached_images.keys())
 
     def get_cached_animation_names(self):
-        '''Get a list of animation filenames that have been cached.
+        """Get a list of animation filenames that have been cached.
 
         This is useful for debugging and profiling only.
 
         :rtype: list
         :return: List of str
-        '''
+        """
         self._require_index()
         return list(self._cached_animations.keys())
 
     def get_texture_bins(self):
-        '''Get a list of texture bins in use.
+        """Get a list of texture bins in use.
 
         This is useful for debugging and profiling only.
 
         :rtype: list
-        :return: List of `TextureBin`
-        '''
+        :return: List of :py:class:`~pyglet.image.atlas.TextureBin`
+        """
         self._require_index()
         return list(self._texture_atlas_bins.values())
 
     def media(self, name, streaming=True):
-        '''Load a sound or video resource.
+        """Load a sound or video resource.
 
         The meaning of `streaming` is as for `media.load`.  Compressed
         sources cannot be streamed (that is, video and compressed audio
@@ -617,14 +660,13 @@ class Loader(object):
                 it should be entirely decoded into memory immediately.
 
         :rtype: `media.Source`
-        '''
+        """
         self._require_index()
         from pyglet import media
         try:
             location = self._index[name]
             if isinstance(location, FileLocation):
-                # Don't open the file if it's streamed from disk -- AVbin
-                # needs to do it.
+                # Don't open the file if it's streamed from disk
                 path = os.path.join(location.path, name)
                 return media.load(path, streaming=streaming)
             else:
@@ -634,10 +676,10 @@ class Loader(object):
             raise ResourceNotFoundException(name)
 
     def texture(self, name):
-        '''Load a texture.
+        """Load a texture.
 
         The named image will be loaded as a single OpenGL texture.  If the
-        dimensions of the image are not powers of 2 a `TextureRegion` will
+        dimensions of the image are not powers of 2 a :py:class:`~pyglet.image.TextureRegion` will
         be returned.
 
         :Parameters:
@@ -645,7 +687,7 @@ class Loader(object):
                 Filename of the image resource to load.
 
         :rtype: `Texture`
-        '''
+        """
         self._require_index()
         if name in self._cached_textures:
             return self._cached_textures[name]
@@ -655,21 +697,36 @@ class Loader(object):
         self._cached_textures[name] = texture
         return texture
 
+    def model(self, name, batch=None):
+        """Load a 3D model.
+
+        :Parameters:
+            `name` : str
+                Filename of the 3D model to load.
+            `batch` : Batch or None
+                An optional Batch instance to add this model to.
+
+        :rtype: `Model`
+        """
+        self._require_index()
+        abspathname = os.path.join(os.path.abspath(self.location(name).path), name)
+        return pyglet.model.load(filename=abspathname, file=self.file(name), batch=batch)
+
     def html(self, name):
-        '''Load an HTML document.
+        """Load an HTML document.
 
         :Parameters:
             `name` : str
                 Filename of the HTML resource to load.
 
         :rtype: `FormattedDocument`
-        '''
+        """
         self._require_index()
         file = self.file(name)
-        return pyglet.text.decode_html(file.read(), self.location(name))
+        return pyglet.text.load(name, file, 'text/html')
 
     def attributed(self, name):
-        '''Load an attributed text document.
+        """Load an attributed text document.
 
         See `pyglet.text.formats.attributed` for details on this format.
 
@@ -678,32 +735,33 @@ class Loader(object):
                 Filename of the attribute text resource to load.
 
         :rtype: `FormattedDocument`
-        '''
+        """
         self._require_index()
         file = self.file(name)
         return pyglet.text.load(name, file, 'text/vnd.pyglet-attributed')
 
     def text(self, name):
-        '''Load a plain text document.
+        """Load a plain text document.
 
         :Parameters:
             `name` : str
                 Filename of the plain text resource to load.
 
         :rtype: `UnformattedDocument`
-        '''
+        """
         self._require_index()
         file = self.file(name)
         return pyglet.text.load(name, file, 'text/plain')
 
     def get_cached_texture_names(self):
-        '''Get the names of textures currently cached.
+        """Get the names of textures currently cached.
 
         :rtype: list of str
-        '''
+        """
         self._require_index()
         return list(self._cached_textures.keys())
 
+
 #: Default resource search path.
 #:
 #: Locations in the search path are searched in order and are always
@@ -714,15 +772,18 @@ class Loader(object):
 #: :type: list of str
 path = []
 
+
 class _DefaultLoader(Loader):
-    def _get_path(self):
+
+    @property
+    def path(self):
         return path
 
-    def _set_path(self, value):
+    @path.setter
+    def path(self, value):
         global path
         path = value
 
-    path = property(_get_path, _set_path)
 
 _default_loader = _DefaultLoader()
 reindex = _default_loader.reindex
@@ -731,12 +792,13 @@ location = _default_loader.location
 add_font = _default_loader.add_font
 image = _default_loader.image
 animation = _default_loader.animation
-get_cached_image_names = _default_loader.get_cached_image_names
-get_cached_animation_names = _default_loader.get_cached_animation_names
-get_texture_bins = _default_loader.get_texture_bins
+model = _default_loader.model
 media = _default_loader.media
 texture = _default_loader.texture
 html = _default_loader.html
 attributed = _default_loader.attributed
 text = _default_loader.text
 get_cached_texture_names = _default_loader.get_cached_texture_names
+get_cached_image_names = _default_loader.get_cached_image_names
+get_cached_animation_names = _default_loader.get_cached_animation_names
+get_texture_bins = _default_loader.get_texture_bins
