@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2019 pyglet contributors
+# Copyright (c) 2008-2020 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -35,8 +35,34 @@
 
 """Mouse constants and utilities for pyglet.window.
 """
-__docformat__ = 'restructuredtext'
-__version__ = '$Id$'
+
+
+class MouseStateHandler(dict):
+    """Simple handler that tracks the state of buttons from the mouse. If a
+    button is pressed then this handler holds a True value for it.
+
+    For example::
+
+        >>> win = window.Window
+        >>> mousebuttons = mouse.MouseStateHandler()
+        >>> win.push_handlers(mousebuttons)
+
+        # Hold down the "left" button...
+
+        >>> mousebuttons[mouse.LEFT]
+        True
+        >>> mousebuttons[mouse.RIGHT]
+        False
+
+    """
+    def on_mouse_press(self, x, y, button, modifiers):
+        self[button] = True
+        
+    def on_mouse_release(self, x, y, button, modifiers):
+        self[button] = False
+        
+    def __getitem__(self, key):
+        return self.get(key, False)
 
 
 def buttons_string(buttons):
@@ -62,7 +88,8 @@ def buttons_string(buttons):
         button_names.append('RIGHT')
     return '|'.join(button_names)
 
+
 # Symbolic names for the mouse buttons
-LEFT =   1 << 0
+LEFT = 1 << 0
 MIDDLE = 1 << 1
-RIGHT =  1 << 2
+RIGHT = 1 << 2
